#INCLUDE "appevent.CH"
#INCLUDE "gra.CH"
#INCLUDE "xbp.CH"
#INCLUDE "dcdialog.CH"
#INCLUDE "dll.CH"

#COMMAND MSGBOXCUSTOM [TITLE <title>] [TEXT <text>] [SIZE <width> [, <height>]] ;
         [TO <status>] => ;
      [<status>] := MsgBoxCustom( [<title>], [<width>], [<height>], [<text>] )

STATIC aDllLoaded := {}

FUNCTION Main()

LOCAL cText, lStatus

TEXT INTO cText WRAP
This is a custom message box. It uses the "Owner-Drawing" feature of the XbpStatic() class.

This implementation shows how to create your own custom message boxes using the eXpress++ method.
ENDTEXT

MSGBOXCUSTOM TITLE 'Hello out there!!' TEXT cText SIZE 60,15 TO lStatus

IF lStatus
  lStatus := MsgBoxCustom( 'Thank you!', 60, 10, 'You are a Gem!' )
ENDIF

RETURN lStatus

* ----------

PROC appsys ; RETURN


* ----------

FUNCTION MsgBoxCustom( cTitle, nWidth, nHeight, cText )

LOCAL GetList[0], oDrawingArea, oDlg, oMsgBox, lStatus

DEFAULT nWidth := 60, ;
        nHeight := 10

@ 0,0 DCDIALOG oDlg DRAWINGAREA oDrawingArea SIZE nWidth, nHeight ;
      NOTITLEBAR BORDER XBPDLG_NO_BORDER INVISIBLE ;
      EVAL {|o,w,h|DC_CenterObject(o), ;
                   DC_SetRoundRectRegion( oDlg, 20 )}

@ 0,0 DCSTATIC TYPE XBPSTATIC_TYPE_TEXT ;
      SIZE DCGUI_PARENTWIDTH, DCGUI_PARENTHEIGHT ;
      SUBCLASS "MsgBoxCustomStatic()" ;
      OBJECT oMsgBox ;
      PARENT oDrawingArea ;
      CAPTION cTitle ;
      PREEVAL {|o|o:titleCaption := o:caption, ;
                  o:lbDown := {|a,b,o|o:moveWindow(a,b)}, ;
                  o:lbClick := {|a,b,o|o:closeWindow(a,b)}}

@ .3, nWidth - 5 DCPUSHBUTTONXP CAPTION 'r' ;
      SIZE 4,1 ;
      FONT '16.Marlett' ACTION {||DC_ReadGuiEvent(DCGUI_EXIT_ABORT,GetList)} ;
      MOUSECOLOR GRA_CLR_WHITE, GRA_CLR_RED ;
      PARENT oMsgBox

@ nHeight - 2.5, nWidth/2 - 5 DCPUSHBUTTONXP CAPTION '&Ok' ;
      ACTION {||DC_ReadGuiEvent(DCGUI_EXIT_OK,GetList)} ;
      SIZE 10, 1.5 ;
      PARENT oMsgBox ;
      FONT '14.Arial Bold' ;
      RADIUS 10 ;
      MOUSESCALE 1.2 MOUSEFONT '20.Arial Bold'

@ 3,2 DCSAY cText SAYWORDBREAK ;
      SIZE nWidth - 4, nHeight - 7 ;
      PARENT oMsgBox ;
      COLOR GRA_CLR_RED, XBPSYSCLR_TRANSPARENT ;
      FONT '14.Arial Bold' ;
      EVAL {||oDlg:show()}

DCREAD GUI PARENT AppDeskTop() TO lStatus

oDlg:destroy()

RETURN lStatus

* ----------

CLASS MsgBoxCustomStatic FROM DC_XbpStatic

PROTECTED:

VAR titleCloseFontObject

EXPORTED:

VAR titleColor
VAR titleHeight
VAR fillColor
VAR radius
VAR gradientStep
VAR titleCaption
VAR titleFont
VAR messageFont
VAR messageColorFG
VAR messageArray
VAR messageFontObject
VAR titleFontObject

* ----------

INLINE METHOD Init(a,b,c,d,e,f,g,h)

::titleColor := { 230, 230, 230 }
::titleHeight := 30
::titleFont := '12.Arial'
::fillColor := { 200, 245, 245 }
::radius := 20
::gradientStep := -2
::titleCaption := 'Testing'
::DC_XbpStatic:init(a,b,c,d,e,f,g,h)
::DC_XbpStatic:drawMode := XBP_DRAW_OWNER
::messageArray := {}
::messageFont := '16.Arial'
::messageColorFG := GRA_CLR_BLACK

RETURN self

* ----------

INLINE METHOD Create(a,b,c,d,e,f,g,h)

IF !Empty(::titleFont) .AND. Empty(::titleFontObject)
  ::titleFontObject := XbpFont():new()
  ::titleFontObject:create(::titleFont)
ENDIF
::titleCloseFontObject := XbpFont():new()
::titleCloseFontObject:create('16.Marlett')
IF !Empty(::messageFont) .AND. Empty(::messageFontObject)
  ::messageFontObject := XbpFont():new()
  ::messageFontObject:create(::messageFont)
ENDIF

::XbpStatic:create(a,b,c,d,e,f,g,h)

RETURN self

* -----------

INLINE METHOD Draw( oPS, aInfo )

LOCAL aStringAttr, i, oFont, aPos, aRect, aSize, nTmp
LOCAL aColors, nColor
LOCAL aStartPos := {aInfo[4][1],aInfo[4][2]}
LOCAL aEndPos   := {aInfo[4][3],aInfo[4][4]}
LOCAL aStartPosSave, aEndPosSave
LOCAL aAttrArea := Array(GRA_AA_COUNT)
LOCAL oSaveFont, nStartRow, nStartCol, nTextHeight, nEndCol

aColors := AClone(::titleColor)

aStartPosSave := AClone(aStartPos)
aEndPosSave := AClone(aEndPos)

FOR i := 1 TO ::titleHeight
  aColors[1] += ::gradientStep
  aColors[2] += ::gradientStep
  aColors[3] += ::gradientStep
  GraSetColor( oPS, GraMakeRGBColor(aColors))
  GraBox( oPS, aStartPos, aEndPos, GRA_OUTLINEFILL, ::radius, ::radius )
  aStartPos[2]++
  aEndPos[2]--
NEXT

aStartPos := aStartPosSave
aEndPos := aEndPosSave
aEndPos[2] -= ::titleHeight

GraSetColor( oPS, GraMakeRGBColor( ::fillColor ))
GraBox( oPS, aStartPos, aEndPos, GRA_OUTLINEFILL, ::radius, ::radius)
aStartPos[2] += ::titleHeight
GraBox( oPS, aStartPos, aEndPos, GRA_OUTLINEFILL, 0, 0)

aStringAttr := Array( GRA_AS_COUNT )
aStringAttr[GRA_AS_COLOR]     := GRA_CLR_BLACK
aStringAttr[GRA_AS_VERTALIGN] := GRA_VALIGN_TOP
GraSetAttrString( oPS, aStringAttr )

IF !Empty(::titleFontObject)
  GraSetFont(oPS,::titleFontObject)
ENDIF
aPos := {20,::CurrentSize()[2]-::titleHeight/2+10}
GraPos( oPS, aPos )
GraStringAt( oPS,,::titleCaption )

/*
oSaveFont := GraSetFont(oPS,::titleCloseFontObject)
aPos := {::CurrentSize()[1]-30,::CurrentSize()[2]-::titleHeight/2+10}
GraPos( oPS, aPos )
GraStringAt( oPS,,'r' )
*/

IF !Empty(::messageArray)
  nStartCol := 10
  nEndCol := ::currentSize()[2]
  GraSetFont(oPS,::messageFontObject)
  aSize := GraQueryTextBox( oPS, ::messageArray[1] )
  nTextHeight := aSize[1,2] - aSize[2,2]
  nStartRow := ::currentSize()[2] - (Len(::messageArray)*nTextHeight) - ::titleHeight

  FOR i := Len(::messageArray) TO 1 STEP -1
      GraCaptionStr( oPS, { nStartCol, nStartRow },, ::messageArray[i] )
      nStartRow += nTextHeight
  NEXT
ENDIF

GraSetFont(oPS,oSaveFont)

RETURN self

* ----------

INLINE METHOD CloseWindow( mp1, mp2 )

IF mp1[1] < ::currentSize()[1] - 30
  RETURN self
ELSEIF mp1[2] < ::currentSize()[2] - ::titleHeight
  RETURN self
ENDIF

PostAppEvent(xbeP_Keyboard, xbeK_ESC,,self)

RETURN self

* ----------

INLINE METHOD MoveWindow( mp1, mp2 )

LOCAL oParent := self, oDeskTop := AppDeskTop(), nEvent, aPos, ;
      aPosXbp, oXbp

IF mp1[1] > ::currentSize()[1] - 30
  RETURN self
ELSEIF mp1[2] < ::currentSize()[2] - ::titleHeight
  RETURN self
ENDIF

RETURN DC_MoveParentWindow( mp1, mp2, self )

* ---------

ENDCLASS

FUNCTION DC_SetRoundRectRegion( oXbp, nRadius ) // nWidth, nHeight, nRadius )

LOCAL nDLL, nDLL2, hRgn, nWidth, nHeight

nWidth := oXbp:currentSize()[1]
nHeight := oXbp:currentSize()[2]

DEFAULT nRadius := 10

nDLL := _DLLLoadLS( 'GDI32.DLL' )
IF nDll <> 0
  hRgn := DllCall( nDLL, DLL_STDCALL,"CreateRoundRectRgn", ;
            0,;
            0,;
            nWidth,;
            nHeight, ;
            nRadius, ;
            nRadius )

  nDll2 := _DLLLoadLs( 'USER32.DLL' )
  DllCall( nDll2, DLL_STDCALL,"SetWindowRgn", oXbp:GetHwnd(), hRgn, 1 )
  _DllUnloadLS(nDll)
  _DLLUnloadLs(nDll2)
ENDIF

RETURN nil

* ---------------

STATIC FUNCTION _DllLoadLS( cDLL, lNoUnload )

  LOCAL nHandle, nX

  DEFAULT lNoUnload := .f.
  cDLL := upper( cDLL )
  if ( nX := ascan( aDLLLoaded, { |aV| aV[1]== cDLL } ) ) > 0
    nHandle := aDLLLoaded[ nX, 2 ]
    if ! lNoUnLoad
      aDLLLoaded[ nX, 3 ] ++
    endif
  else
    nHandle := DLLLoad( cDLL )
    if nHandle <> 0      //added if statement, if dll load fails, alows another attempt at it.
      aadd( aDllLoaded, { cDLL, nHandle, 1 } )
    endif
  endif

return nHandle

* ---------------

STATIC FUNCTION _UnLoadAllLS()

  LOCAL i
  FOR i := 1 to len( aDllLoaded )
    DLLUnload( aDLLLoaded[ i, 2 ] )
  NEXT

RETURN nil

* ---------------

STATIC FUNCTION _DllUnLoadLS( nHandle )

  local nX
  if ( nX := ascan( aDLLLoaded, {|aV| aV[2] == nHandle } ) )> 0
    aDLLLoaded[ nX, 3 ] --
    if aDLLLoaded[ nX, 3 ] <= 0
      adel( aDLLLoaded, nX )
      asize( aDLLLoaded, len( aDLLLoaded ) -1 )
      DLLUnload( nHandle )
    endif
  else
    DLLUnload( nHandle )
  endif

RETURN nil


